@@ -38,8 +38,7 @@ from tempfile import NamedTemporaryFile
 from typing import Iterable, Tuple
 
 from PyQt5.QtCore import (
-    Qt, QMimeData, QModelIndex, QBuffer, QRect, QRectF, QSize,
-    QItemSelectionModel)
+    Qt, QMimeData, QModelIndex, QBuffer, QRect, QRectF, QItemSelectionModel)
 from PyQt5.QtGui import QTextDocument, QImage, QPainter, QBrush, QPen
 from PyQt5.QtWidgets import (
     QApplication, QMessageBox, QInputDialog, QStyleOptionViewItem, QTableView)
@@ -194,7 +193,10 @@ class Workflows:
             return
 
         # Set current cell to upper left corner
-        self.main_window.grid.current = 0, 0, 0
+        for grid in self.main_window.grids:
+            grid.current = 0, 0, 0
+            # Select upper left cell because initial selection behaves strange
+            grid.reset_selection()
 
         # Reset grid
         self.main_window.grid.model.reset()
@@ -205,17 +207,15 @@ class Workflows:
         # Set new shape
         self.main_window.grid.model.shape = shape
 
-        # Select upper left cell because initial selection behaves strange
-        self.main_window.grid.reset_selection()
-
         # Update cell spans and zoom because this is unsupported by the model
-        with self.main_window.grid.undo_resizing_row():
-            with self.main_window.grid.undo_resizing_column():
-                self.main_window.grid.update_cell_spans()
-                self.main_window.grid.update_zoom()
+        for grid in self.main_window.grids:
+            with grid.undo_resizing_row():
+                with grid.undo_resizing_column():
+                    grid.update_cell_spans()
+                    grid.update_zoom()
 
-        # Update index widgets
-        self.main_window.grid.update_index_widgets()
+            # Update index widgets
+            grid.update_index_widgets()
 
         # Change the main window filepath state
         self.main_window.settings.changed_since_save = False
@@ -320,16 +320,17 @@ class Workflows:
         grid.model.shape = shape
 
         # Update cell spans and zoom because this is unsupported by the model
-        with self.main_window.grid.undo_resizing_row():
-            with self.main_window.grid.undo_resizing_column():
-                self.main_window.grid.update_cell_spans()
-                self.main_window.grid.update_zoom()
+        for grid in self.main_window.grids:
+            with grid.undo_resizing_row():
+                with grid.undo_resizing_column():
+                    grid.update_cell_spans()
+                    grid.update_zoom()
 
-        # Update index widgets
-        grid.update_index_widgets()
+            # Update index widgets
+            grid.update_index_widgets()
 
-        # Select upper left cell because initial selection behaves strangely
-        grid.reset_selection()
+            # Select upper left cell because initial selection oddities
+            grid.reset_selection()
 
         # Change the main window last input directory state
         self.main_window.settings.last_file_input_path = filepath
@@ -472,6 +473,12 @@ class Workflows:
         window_title = "{filename} - pyspread".format(filename=filepath.name)
         self.main_window.setWindowTitle(window_title)
 
+        # Add to file history
+        self.main_window.settings.add_to_file_history(filepath.as_posix())
+
+        # Update recent files in the file menu
+        self.main_window.menuBar().file_menu.history_submenu.update()
+
         self.sign_file(filepath)
 
     def file_save(self):
@@ -545,8 +552,9 @@ class Workflows:
             keep_header = dialect.hasheader and dialect.keepheader
         except AttributeError:
             keep_header = False
-        row, column, _ = current = self.main_window.grid.current
-        model = self.main_window.grid.model
+        grid = self.main_window.focused_grid
+        row, column, _ = current = grid.current
+        model = grid.model
         rows, columns, tables = model.shape
 
         # Dialog accepted, now check if grid is large enough
@@ -670,8 +678,10 @@ class Workflows:
         # Determine what filters ae available
         filters_list = ["CSV (*.csv)"]
 
-        current = self.main_window.grid.current
-        code_array = self.main_window.grid.model.code_array
+        grid = self.main_window.focused_grid
+
+        current = grid.current
+        code_array = grid.model.code_array
 
         res = code_array[current]
 
@@ -723,15 +733,16 @@ class Workflows:
 
         """
 
+        grid = self.main_window.focused_grid
+
         # Get area for csv export
-        area = CsvExportAreaDialog(self.main_window,
-                                   self.main_window.grid,
+        area = CsvExportAreaDialog(self.main_window, grid,
                                    title="Csv export area").area
         if area is None:
             return
 
-        code_array = self.main_window.grid.model.code_array
-        table = self.main_window.grid.table
+        code_array = grid.model.code_array
+        table = grid.table
         csv_data = code_array[area.top: area.bottom + 1,
                               area.left: area.right + 1, table]
 
@@ -755,8 +766,9 @@ class Workflows:
 
         """
 
-        code_array = self.main_window.grid.model.code_array
-        qimage = code_array[self.main_window.grid.current]
+        grid = self.main_window.focused_grid
+        code_array = grid.model.code_array
+        qimage = code_array[grid.current]
 
         try:
             if not qimage.save(filepath, file_format):
@@ -777,8 +789,9 @@ class Workflows:
             # matplotlib is not installed
             return
 
-        code_array = self.main_window.grid.model.code_array
-        figure = code_array[self.main_window.grid.current]
+        grid = self.main_window.focused_grid
+        code_array = grid.model.code_array
+        figure = code_array[grid.current]
 
         try:
             figure.savefig(filepath, format=file_format)
@@ -793,10 +806,11 @@ class Workflows:
 
         """
 
-        __zoom = self.main_window.grid.zoom
-        self.main_window.grid.zoom = zoom
+        grid = self.main_window.focused_grid
+        __zoom = grid.zoom
+        grid.zoom = zoom
         yield
-        self.main_window.grid.zoom = __zoom
+        grid.zoom = __zoom
 
     def get_paint_rows(self, top: int, bottom: int) -> Iterable[int]:
         """Iterator of rows to paint
@@ -806,13 +820,14 @@ class Workflows:
 
         """
 
-        rows = self.main_window.grid.model.shape[0]
+        grid = self.main_window.focused_grid
+        rows = grid.model.shape[0]
         top = max(0, min(rows - 1, top))
         bottom = max(0, min(rows - 1, bottom))
         if top == -1:
             top = 0
         if bottom == -1:
-            bottom = self.main_window.grid.model.shape[0]
+            bottom = grid.model.shape[0]
 
         return range(top, bottom + 1)
 
@@ -824,13 +839,14 @@ class Workflows:
 
         """
 
-        columns = self.main_window.grid.model.shape[1]
+        grid = self.main_window.focused_grid
+        columns = grid.model.shape[1]
         left = max(0, min(columns - 1, left))
         right = max(0, min(columns - 1, right))
         if left == -1:
             left = 0
         if right == -1:
-            right = self.main_window.grid.model.shape[1]
+            right = grid.model.shape[1]
 
         return range(left, right + 1)
 
@@ -842,13 +858,14 @@ class Workflows:
 
         """
 
-        tables = self.main_window.grid.model.shape[2]
+        grid = self.main_window.focused_grid
+        tables = grid.model.shape[2]
         first = max(0, min(tables - 1, first))
         last = max(0, min(tables - 1, last))
         if first == -1:
             first = 0
         if last == -1:
-            last = self.main_window.grid.model.shape[2]
+            last = grid.model.shape[2]
 
         return range(first, last + 1)
 
@@ -860,7 +877,7 @@ class Workflows:
 
         """
 
-        grid = self.main_window.grid
+        grid = self.main_window.focused_grid
         rows = self.get_paint_rows(top, bottom)
         return sum(grid.rowHeight(row) for row in rows)
 
@@ -872,7 +889,7 @@ class Workflows:
 
         """
 
-        grid = self.main_window.grid
+        grid = self.main_window.focused_grid
         columns = self.get_paint_columns(left, right)
         return sum(grid.columnWidth(column) for column in columns)
 
@@ -888,7 +905,7 @@ class Workflows:
 
         """
 
-        grid = self.main_window.grid
+        grid = self.main_window.focused_grid
         code_array = grid.model.code_array
         cell_attributes = code_array.cell_attributes
 
@@ -948,7 +965,7 @@ class Workflows:
 
         """
 
-        grid = self.main_window.grid
+        grid = self.main_window.focused_grid
         model = grid.model
         selection = grid.selection
 
@@ -975,7 +992,7 @@ class Workflows:
 
         """
 
-        grid = self.main_window.grid
+        grid = self.main_window.focused_grid
         table = grid.table
         selection = grid.selection
         bbox = selection.get_grid_bbox(grid.model.shape)
@@ -1095,7 +1112,7 @@ class Workflows:
 
         """
 
-        grid = self.main_window.grid
+        grid = self.main_window.focused_grid
 
         if grid.has_selection():
             self._copy_results_selection(grid)
@@ -1110,7 +1127,7 @@ class Workflows:
 
         """
 
-        grid = self.main_window.grid
+        grid = self.main_window.focused_grid
         model = grid.model
         (top, left), (bottom, right) = selection.get_grid_bbox(model.shape)
         table = grid.table
@@ -1129,9 +1146,11 @@ class Workflows:
                 break
             for column, value in enumerate(cycle(line)):
                 paste_column = column + left
-                if ((paste_row, paste_column, table) in code_array
+                paste_key = paste_row, paste_column, table
+                if (paste_key in code_array
                         and paste_column <= right):
-                    if (paste_row, paste_column) in selection:
+                    if ((paste_row, paste_column) in selection and not
+                            code_array.cell_attributes[paste_key].locked):
                         index = model.index(paste_row, paste_column,
                                             QModelIndex())
                         # Preserve line breaks
@@ -1153,7 +1172,7 @@ class Workflows:
 
         """
 
-        grid = self.main_window.grid
+        grid = self.main_window.focused_grid
         model = grid.model
         top, left, table = current = grid.current
         code_array = grid.model.code_array
@@ -1196,7 +1215,7 @@ class Workflows:
 
         """
 
-        grid = self.main_window.grid
+        grid = self.main_window.focused_grid
 
         clipboard = QApplication.clipboard()
         data = clipboard.text()
@@ -1219,15 +1238,17 @@ class Workflows:
 
         """
 
+        grid = self.main_window.focused_grid
+
         codelines = svg.splitlines()
         codelines[0] = '"""' + codelines[0]
         codelines[-1] = codelines[-1] + '"""'
         code = "\n".join(codelines)
 
-        model = self.main_window.grid.model
+        model = grid.model
         description = "Insert svg image into cell {}".format(index)
 
-        self.main_window.grid.on_image_renderer_pressed(True)
+        grid.on_image_renderer_pressed(True)
         with self.disable_entryline_updates():
             command = commands.SetCellCode(code, model, index, description)
             self.main_window.undo_stack.push(command)
@@ -1251,6 +1272,8 @@ class Workflows:
             for i in range(0, len(string), length):
                 yield string[i:i+length]
 
+        grid = self.main_window.focused_grid
+
         repr_image_data = repr(b85encode(bz2.compress(image_data)))
         newline = "'\n+b'"
 
@@ -1267,10 +1290,10 @@ class Workflows:
 
         code = "\n".join(code_lines)
 
-        model = self.main_window.grid.model
+        model = grid.model
         description = "Insert image into cell {}".format(index)
 
-        self.main_window.grid.on_image_renderer_pressed(True)
+        grid.on_image_renderer_pressed(True)
         with self.disable_entryline_updates():
             command = commands.SetCellCode(code, model, index, description)
             self.main_window.undo_stack.push(command)
@@ -1278,7 +1301,7 @@ class Workflows:
     def edit_paste_as(self):
         """Pastes clipboard into one cell using a user specified mime type"""
 
-        grid = self.main_window.grid
+        grid = self.main_window.focused_grid
         model = grid.model
 
         # The mimetypes that are supported by pyspread
@@ -1357,7 +1380,7 @@ class Workflows:
 
         """
 
-        grid = self.main_window.grid
+        grid = self.main_window.focused_grid
         findnextmatch = grid.model.code_array.findnextmatch
 
         find_editor = find_dialog.search_text_editor
@@ -1403,10 +1426,12 @@ class Workflows:
 
         """
 
+        grid = self.main_window.focused_grid
+
         find_string, next_match = self._get_next_match(find_dialog)
 
         if next_match:
-            self.main_window.grid.current = next_match
+            grid.current = next_match
 
             regexp = find_dialog.regex_checkbox.isChecked()
             self._display_match_msg(find_string, next_match, regexp)
@@ -1417,7 +1442,8 @@ class Workflows:
     def edit_find_next(self):
         """Edit -> Find next workflow"""
 
-        grid = self.main_window.grid
+        grid = self.main_window.focused_grid
+
         findnextmatch = grid.model.code_array.findnextmatch
 
         find_editor = self.main_window.find_toolbar.find_editor
@@ -1461,7 +1487,9 @@ class Workflows:
 
         """
 
-        model = self.main_window.grid.model
+        grid = self.main_window.focused_grid
+
+        model = grid.model
 
         find_string, next_match = self._get_next_match(replace_dialog)
         replace_string = replace_dialog.replace_text_editor.text()
@@ -1477,7 +1505,7 @@ class Workflows:
             command = commands.SetCellCode(new_code, model, index, description)
             self.main_window.undo_stack.push(command)
 
-            self.main_window.grid.current = next_match
+            grid.current = next_match
 
             self.main_window.statusBar().showMessage(description)
 
@@ -1499,8 +1527,10 @@ class Workflows:
     def edit_resize(self):
         """Edit -> Resize workflow"""
 
+        grid = self.main_window.focused_grid
+
         # Get grid shape from user
-        old_shape = self.main_window.grid.model.code_array.shape
+        old_shape = grid.model.code_array.shape
         title = "Resize grid"
         shape = GridShapeDialog(self.main_window, old_shape, title=title).shape
         self._resize_grid(shape)
@@ -1512,8 +1542,9 @@ class Workflows:
 
         """
 
-        grid = self.main_window.grid
-        old_shape = self.main_window.grid.model.code_array.shape
+        grid = self.main_window.focused_grid
+
+        old_shape = grid.model.code_array.shape
 
         # Check if shape is valid
         try:
@@ -1522,7 +1553,7 @@ class Workflows:
             self.main_window.statusBar().showMessage('Error: ' + str(err))
             return
 
-        self.main_window.grid.current = 0, 0, 0
+        grid.current = 0, 0, 0
 
         description = "Resize grid to {}".format(shape)
 
@@ -1531,19 +1562,21 @@ class Workflows:
             self.main_window.undo_stack.push(command)
 
         # Select upper left cell because initial selection behaves strangely
-        self.main_window.grid.reset_selection()
+        grid.reset_selection()
 
     # View menu
 
     def view_goto_cell(self):
         """View -> Go to cell workflow"""
 
+        grid = self.main_window.focused_grid
+
         # Get cell key from user
-        shape = self.main_window.grid.model.shape
+        shape = grid.model.shape
         key = CellKeyDialog(self.main_window, shape).key
 
         if key is not None:
-            self.main_window.grid.current = key
+            grid.current = key
 
     # Format menu
 
@@ -1568,7 +1601,7 @@ class Workflows:
                 except KeyError:
                     pass
 
-        grid = self.main_window.grid
+        grid = self.main_window.focused_grid
         code_array = grid.model.code_array
         cell_attributes = code_array.cell_attributes
 
@@ -1608,7 +1641,7 @@ class Workflows:
         clipboard = QApplication.clipboard()
         mime_data = clipboard.mimeData()
 
-        grid = self.main_window.grid
+        grid = self.main_window.focused_grid
         model = grid.model
 
         row, column, table = grid.current
@@ -1650,7 +1683,7 @@ class Workflows:
     def macro_insert_image(self):
         """Insert image workflow"""
 
-        grid = self.main_window.grid
+        grid = self.main_window.focused_grid
 
         dial = ImageFileOpenDialog(self.main_window)
         if not dial.file_path:
@@ -1686,9 +1719,12 @@ class Workflows:
     def macro_insert_chart(self):
         """Insert chart workflow"""
 
-        code_array = self.main_window.grid.model.code_array
-        code = code_array(self.main_window.grid.current)
-        current = self.main_window.grid.current
+        grid = self.main_window.focused_grid
+
+        model = grid.model
+        code_array = model.code_array
+        current = grid.current
+        code = code_array(current)
 
         if current in self.cell2dialog:
             self.cell2dialog[current].activateWindow()
@@ -1698,21 +1734,19 @@ class Workflows:
         chart_dialog = ChartDialog(self.main_window, current)
         self.cell2dialog[current] = chart_dialog
 
+        chart_dialog.show()
+
         if code is not None:
             chart_dialog.editor.setPlainText(code)
 
-        chart_dialog.show()
-
         if chart_dialog.exec_() == ChartDialog.Accepted:
             code = chart_dialog.editor.toPlainText()
-            self.main_window.grid.current = current
-            index = self.main_window.grid.currentIndex()
-            self.main_window.grid.clearSelection()
-            self.main_window.grid.selectionModel().select(
-                    index, QItemSelectionModel.Select)
-            self.main_window.grid.on_matplotlib_renderer_pressed(True)
+            grid.current = current
+            index = grid.currentIndex()
+            grid.clearSelection()
+            grid.selectionModel().select(index, QItemSelectionModel.Select)
+            grid.on_matplotlib_renderer_pressed(True)
 
-            model = self.main_window.grid.model
             description = "Insert chart into cell {}".format(index)
             command = commands.SetCellCode(code, model, index, description)
 
